#include "xparameters.h"
#include "xbasic_types.h"
#include "xio.h"
#include "set_registers.h"
#include "xuartlite_l.h"

#define REG_START 0x01800000
#define LINE_COUNT 0x01801FFC
#define VERT_SYNC 0x01802FFC
#define FRAME_COUNT 0x018008FC
#define THRESHOLD 0x709000A0

extern void set_registers();

void set_direction(int total[16]){
  int k, sum, average, region,tempdir, temppixel, left, middle, right;
  int direction[16];
  int griddirsum[11],gridpixelsum[11];
  Xuint32 temp_num;
  sum = 0;
  average = 0;
  for (k=0; k<16; k++)
  {
    // When a grid sums up to 50 total pixels or less, I considered that as a misread
    // since a large enough object should contain more than 50 pixels in a grid
    if (total[k]<50)
      total[k] = 0;
    sum = sum + total[k];
    }

  average = sum >> 4;
  sum =0;
  for (k=0; k<16; k++){
   //A misread when average grid contains up to 20 only
   if (average > 20) {
     //A valid object in the grid whenever it has more than 512 pixels
     if ((total[k] >= average) || (total[k] > 0x200)){
          print("o ");
	  direction[k] = 1;
	  sum += 1;
     }
     else {
          print("x ");
          direction[k]= 0;
     }
   }
   else{
      print("x ");
      direction[k] = 0;
   }
   if (k%4 == 3)
     print("\r\n");
 }

 tempdir = 0;
 temppixel = 0;
 //Calculate the sum of each 2*2 block grid
 for (k=0;k<11;k++){
    griddirsum[k] = direction[k]+direction[k+1]+direction[k+4]+direction[k+5];
    gridpixelsum[k] = total[k]+total[k+1]+total[k+4]+total[k+5];
    if (k%4 != 3){
      if ((griddirsum[k] > tempdir) || (griddirsum[k] == tempdir && gridpixelsum[k] > temppixel)){
         tempdir = griddirsum[k];
         temppixel = gridpixelsum[k];
         region = k;
         }
      }
 }

//The print statement will output to serial port to control the robot movement
 left  = griddirsum[0] + griddirsum[8];
 right  = griddirsum[2] + griddirsum[10];
 middle = griddirsum[1] + griddirsum[9];
 if (sum >= 12){
  print ("S\r\n");}
 else if (middle >= 6){
   print("S\r\n");}
 else if(left > right){
   print("L\r\n");}
 else if (right > left){
   print("R\r\n");}
 else if (left + right == 0){
   print("H\r\n");}
 else{
   print("S\r\n");}
}


int write_video_minicom()
{
  int nPixs,k;
  int total[16];
  int count = 0;
  Xuint32 pixel_num;
  int bram_addr = REG_START;

  //Read data from all 16 grids, 4 * 4
  for (nPixs = 0; nPixs<4; nPixs++)
  {

       pixel_num = XIo_In32(bram_addr+0);
       total[count+0] = pixel_num;

       pixel_num = XIo_In32(bram_addr+4);
       total[count+1] = pixel_num;

       pixel_num = XIo_In32(bram_addr+8);
       total[count+2] = pixel_num;

       pixel_num = XIo_In32(bram_addr+12);
       total[count+3] = pixel_num;

       count+=4;
       bram_addr += 16;
       //Delay
       for (k=0; k<10000000;k++);

       if (!XIo_In32(VERT_SYNC)){
	  for( k  = 0; k < 16; k++)
             total[k] = 0;
	  return 0;
	}
  }

  set_direction(total);
  for (k=0; k<50000000;k++);
	return 1;
}


inline void read_one_frame()
{
  int j;
  while (1){
      if (!XIo_In32(VERT_SYNC))
	break;

      //Print pixel counts to minicom
        if(write_video_minicom()){
		for (j=0; j<1250000000;j++);
	}
	print ("\r\n");
      if (!XIo_In32(VERT_SYNC))
	break;
  }
}



int main()
{
  int i;
  microblaze_enable_icache();

  set_registers();
  XIo_Out32 (0x01802DDC, THRESHOLD);

  // Wait for a little bit
  for (i=0; i<10000;i++);

  while (1)
  {
    // Wait for the vertical synchronism
    while ((XIo_In32(VERT_SYNC)));
    while (!(XIo_In32(VERT_SYNC)));

    if (XIo_In32 (FRAME_COUNT))
    {
      while ((XIo_In32(VERT_SYNC)));
      while (!(XIo_In32(VERT_SYNC)));
    }

    read_one_frame();

  }
 return 0;
}

